@@ -3,15 +3,13 @@ using Gtk;
 using System.Threading;
 
 namespace FSpot {
-	public class ColorDialog {
+	public class ColorDialog : GladeDialog {
 		Gdk.Pixbuf ScaledPixbuf;
 		Gdk.Pixbuf AdjustedPixbuf;
-		
-		Delay expose_timeout;
 
-		[Glade.Widget] private Gtk.Dialog external_color_dialog;
-		[Glade.Widget] private Gtk.Dialog inline_color_dialog;
-		
+#if USE_THREAD		
+		Delay expose_timeout;
+#endif
 		[Glade.Widget] private Gtk.SpinButton source_spinbutton;
 		[Glade.Widget] private Gtk.SpinButton dest_spinbutton;
 
@@ -38,6 +36,9 @@ namespace FSpot {
 		
 		private void Adjust ()
 		{
+			if (brightness_scale == null)
+				return;
+
 			Cms.Profile srgb = Cms.Profile.CreateSRgb ();
 			Cms.Profile bchsw = Cms.Profile.CreateAbstract (10, brightness_scale.Value,
 									contrast_scale.Value,
@@ -50,18 +51,17 @@ namespace FSpot {
 			next_transform = new Cms.Transform (list, 
 							    PixbufUtils.PixbufCmsFormat (AdjustedPixbuf),
 							    PixbufUtils.PixbufCmsFormat (AdjustedPixbuf),
-							    Cms.Intent.Perceptual, 0x0100);
+							    Cms.Intent.Perceptual, 0x0000);
 			
 			lock (AdjustedPixbuf) {
 				PixbufUtils.ColorAdjust (ScaledPixbuf,
 							 AdjustedPixbuf,
-							 brightness_scale.Value,
-							 contrast_scale.Value,
-							 hue_scale.Value,
-							 sat_scale.Value,
-							 source_spinbutton.ValueAsInt,
-							 dest_spinbutton.ValueAsInt);
+							 next_transform);
+#if USE_THREAD
 				expose_timeout.Start ();
+#else
+				this.QueueDraw ();
+#endif
 			}
 		}
 			
@@ -82,22 +82,36 @@ namespace FSpot {
 		
 		public void HandleDestroyEvent (object sender, DestroyEventArgs arg)
 		{
+#if USE_THREAD
 			expose_timeout.Stop ();
+#endif
 		}
 		
 		public void RangeChanged (object sender, EventArgs args)
 		{
+			if (!view.CurrentPhotoValid ())
+				return;
+
+#if USE_THREAD
 			if (thread != null && thread.IsAlive)
 				thread.Abort ();
 			
+			
 			thread = new Thread (new ThreadStart (Adjust));
 			thread.Start ();
+#else
+			Adjust ();
+#endif
 		}
 		
 		public void Save ()
 		{
+			if (!view.CurrentPhotoValid ())
+				return;
+
 			Console.WriteLine ("Saving....");
 			Photo photo = view.Query.Photos[view.CurrentPhoto];
+			Exif.ExifData data = new Exif.ExifData (photo.DefaultVersionPath);
 			
 			uint version = photo.DefaultVersionId;
 			if (version == Photo.OriginalVersionId) {
@@ -117,19 +131,19 @@ namespace FSpot {
 			
 			try {
 				string version_path = photo.GetVersionPath (version);
-				
-				final.Savev (version_path, "jpeg", null, null);
+
+				PixbufUtils.SaveJpeg (final, version_path, 95, data);
 				ThumbnailGenerator.Create (version_path).Dispose ();
 				photo.DefaultVersionId = version;
 				view.Query.Commit (view.CurrentPhoto);
 			} catch (GLib.GException ex) {
 				// FIXME error dialog.
-				Console.WriteLine ("error {0}", ex);
+				Console.WriteLine (ex.ToString ());
 			}
 			
 			Console.WriteLine ("Saving....");
-			color_dialog.Sensitive = false;
-			color_dialog.Destroy ();
+			this.Dialog.Sensitive = false;
+			this.Dialog.Destroy ();
 		}
 		
 		public void Cancel ()
@@ -137,7 +151,7 @@ namespace FSpot {
 			view.Transform = null;
 			view.QueueDraw ();
 			view.PhotoChanged -= HandlePhotoChanged;
-			color_dialog.Destroy ();
+			this.Dialog.Destroy ();
 		}
 		
 		private void HandleOkClicked (object sender, EventArgs args)
@@ -150,9 +164,11 @@ namespace FSpot {
 
 		private void HandlePhotoChanged (PhotoImageView view)
 		{
-			AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
-			ScaledPixbuf = AdjustedPixbuf.Copy ();			
-			RangeChanged (null, null);
+			if (view.CurrentPhotoValid ()) {
+				AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
+				ScaledPixbuf = AdjustedPixbuf.Copy ();			
+				RangeChanged (null, null);
+			}
 		}
 		
 		private void HandleCancelClicked (object sender, EventArgs args)
@@ -160,16 +176,6 @@ namespace FSpot {
 			Cancel ();
 		}
 		
-		private Gtk.Dialog color_dialog
-		{
-			get {
-				if (external_color_dialog != null)
-					return external_color_dialog;
-				else
-					return inline_color_dialog;
-			}
-		}
-		
 		public ColorDialog (FSpot.PhotoQuery query, int item)
 		{
 			view = new FSpot.PhotoImageView (query);
@@ -177,38 +183,33 @@ namespace FSpot {
 			view.Show ();
 			view.CurrentPhoto = item;
 
-			AttachInterface ("external_color_dialog");
+			this.CreateDialog ("external_color_dialog");
+			AttachInterface ();
 		}
 
 		public ColorDialog (FSpot.PhotoImageView view)       
 		{
 			this.view = view;
-			AttachInterface ("inline_color_dialog");
+			this.CreateDialog ("inline_color_dialog");
+			AttachInterface ();
 		}
 
-		private void AttachInterface (string ui_path)
+		private void AttachInterface ()
 		{
-			Glade.XML xml = new Glade.XML (null, "f-spot.glade", ui_path, null);
-
 			view.PhotoChanged += HandlePhotoChanged;
-			xml.Autoconnect (this);
-			
-			AdjustedPixbuf = PhotoLoader.LoadAtMaxSize (view.Query.Photos [view.CurrentPhoto], 300, 300);
-			ScaledPixbuf = AdjustedPixbuf.Copy ();
-
 			hist = new FSpot.Histogram ();
+#if USE_THREAD
 			expose_timeout = new FSpot.Delay (new GLib.IdleHandler (this.QueueDraw));
-
+#endif
 
 			#if true
-			Gdk.Color c = color_dialog.Style.Backgrounds [(int)Gtk.StateType.Active];
+			Gdk.Color c = this.Dialog.Style.Backgrounds [(int)Gtk.StateType.Active];
 			hist.Color [0] = (byte) (c.Red / 0xff);
 			hist.Color [1] = (byte) (c.Green / 0xff);
 			hist.Color [2] = (byte) (c.Blue / 0xff);
 			hist.Color [3] = 0xff;
 			#endif
 
-			hist.FillValues (AdjustedPixbuf);
 			histogram_image.Pixbuf = hist.GeneratePixbuf ();
 
 			brightness_spinbutton.Adjustment = brightness_scale.Adjustment;
@@ -231,6 +232,8 @@ namespace FSpot {
 			sat_scale.ValueChanged += RangeChanged;
 			source_spinbutton.ValueChanged += RangeChanged;
 			dest_spinbutton.ValueChanged += RangeChanged;
+
+			HandlePhotoChanged (view);
 		}
 	}
 }
